@@ -1,13 +1,14 @@
 # This file is part of Tryton.  The COPYRIGHT file at the top level of
 # this repository contains the full copyright notices and license terms.
-from trytond.model import ModelView, ModelSQL, fields
-from trytond.pyson import Eval
 from trytond import backend
-
-__all__ = ['Country', 'Subdivision', 'Zip']
+from trytond.model import ModelView, ModelSQL, DeactivableMixin, fields
+from trytond.pool import Pool
+from trytond.pyson import Eval
+from trytond.transaction import Transaction
+from trytond.tools import lstrip_wildcard
 
 
-class Country(ModelSQL, ModelView):
+class Country(DeactivableMixin, ModelSQL, ModelView):
     'Country'
     __name__ = 'country.country'
     name = fields.Char('Name', required=True, translate=True,
@@ -28,11 +29,15 @@ class Country(ModelSQL, ModelView):
 
     @classmethod
     def __register__(cls, module_name):
-        TableHandler = backend.get('TableHandler')
-        table = TableHandler(cls, module_name)
+        pool = Pool()
+        Data = pool.get('ir.model.data')
+        data = Data.__table__()
+        cursor = Transaction().connection.cursor()
 
         super(Country, cls).__register__(module_name)
 
+        table = cls.__table_handler__(module_name)
+
         # Migration from 3.4: drop unique constraints from name and code
         table.drop_constraint('name_uniq')
         table.drop_constraint('code_uniq')
@@ -40,13 +45,20 @@ class Country(ModelSQL, ModelView):
         # Migration from 3.8: remove required on code
         table.not_null_action('code', 'remove')
 
+        # Migration from 5.2: remove country data
+        cursor.execute(*data.delete(where=(data.module == 'country')
+                & (data.model == cls.__name__)))
+
     @classmethod
     def search_rec_name(cls, name, clause):
+        code_value = clause[2]
+        if clause[1].endswith('like'):
+            code_value = lstrip_wildcard(clause[2])
         return ['OR',
             ('name',) + tuple(clause[1:]),
-            ('code',) + tuple(clause[1:]),
-            ('code3',) + tuple(clause[1:]),
-            ('code_numeric',) + tuple(clause[1:]),
+            ('code', clause[1], code_value) + tuple(clause[3:]),
+            ('code3', clause[1], code_value) + tuple(clause[3:]),
+            ('code_numeric', clause[1], code_value) + tuple(clause[3:]),
             ]
 
     @classmethod
@@ -71,7 +83,7 @@ class Country(ModelSQL, ModelView):
         super(Country, cls).write(*args)
 
 
-class Subdivision(ModelSQL, ModelView):
+class Subdivision(DeactivableMixin, ModelSQL, ModelView):
     "Subdivision"
     __name__ = 'country.subdivision'
     country = fields.Many2One('country.country', 'Country',
@@ -149,8 +161,11 @@ class Subdivision(ModelSQL, ModelView):
         ('metropolitan region', 'Metropolitan region'),
         ('municipalities', 'Municipalities'),
         ('municipality', 'Municipality'),
+        ('nation', 'Nation'),
         ('oblast', 'Oblast'),
         ('outlying area', 'Outlying area'),
+        ('overseas department', 'Overseas department'),
+        ('overseas region', 'Overseas region'),
         ('overseas region/department', 'Overseas region/department'),
         ('overseas territorial collectivity',
             'Overseas territorial collectivity'),
@@ -196,6 +211,19 @@ class Subdivision(ModelSQL, ModelView):
         super(Subdivision, cls).__setup__()
         cls._order.insert(0, ('code', 'ASC'))
 
+    @classmethod
+    def __register__(cls, module_name):
+        pool = Pool()
+        Data = pool.get('ir.model.data')
+        data = Data.__table__()
+        cursor = Transaction().connection.cursor()
+
+        super().__register__(module_name)
+
+        # Migration from 5.2: remove country data
+        cursor.execute(*data.delete(where=(data.module == 'country')
+                & (data.model == cls.__name__)))
+
     @classmethod
     def search_rec_name(cls, name, clause):
         return ['OR',
@@ -223,31 +251,41 @@ class Subdivision(ModelSQL, ModelView):
         super(Subdivision, cls).write(*args)
 
 
-class Zip(ModelSQL, ModelView):
-    "Zip"
-    __name__ = 'country.zip'
+class PostalCode(ModelSQL, ModelView):
+    "Postal Code"
+    __name__ = 'country.postal_code'
     country = fields.Many2One('country.country', 'Country', required=True,
         select=True, ondelete='CASCADE',
-        help="The country where the zip is.")
+        help="The country that contains the postal code.")
     subdivision = fields.Many2One('country.subdivision', 'Subdivision',
         select=True, ondelete='CASCADE',
         domain=[('country', '=', Eval('country', -1))],
         depends=['country'],
-        help="The subdivision where the zip is.")
-    zip = fields.Char('Zip')
-    city = fields.Char('City', help="The name of the city for the zip.")
+        help="The subdivision where the postal code is.")
+    postal_code = fields.Char('Postal Code')
+    city = fields.Char(
+        "City", help="The city associated with the postal code.")
 
     @classmethod
     def __setup__(cls):
-        super(Zip, cls).__setup__()
+        super().__setup__()
         cls._order.insert(0, ('country', 'ASC'))
-        cls._order.insert(0, ('zip', 'ASC'))
+        cls._order.insert(0, ('postal_code', 'ASC'))
+
+    @classmethod
+    def __register__(cls, module):
+        # Migration from 5.8: rename zip to postal_code
+        backend.TableHandler.table_rename('country_zip', cls._table)
+        table_h = cls.__table_handler__(module)
+        table_h.column_rename('zip', 'postal_code')
+
+        super().__register__(module)
 
     def get_rec_name(self, name):
-        if self.city and self.zip:
-            return '%s (%s)' % (self.city, self.zip)
+        if self.city and self.postal_code:
+            return '%s (%s)' % (self.city, self.postal_code)
         else:
-            return (self.zip or self.city or str(self.id))
+            return (self.postal_code or self.city or str(self.id))
 
     @classmethod
     def search_rec_name(cls, name, clause):
@@ -256,6 +294,6 @@ class Zip(ModelSQL, ModelView):
         else:
             bool_op = 'OR'
         return [bool_op,
-            ('zip',) + tuple(clause[1:]),
+            ('postal_code',) + tuple(clause[1:]),
             ('city',) + tuple(clause[1:]),
             ]
